# Display and run all Koenig SOF files in a folder chosen by the user.
# Author: Dave Greene (dvgrn@juno.com), 29 May 2006, revised 13 June.
# TODO: display standard form of objects.  Objects were reflected across the
#       \ diagonal only because that made it easier to scroll upward, and
#       scrolling upward allowed the first n ON cells to be removed from the
#       current cell list with every cycle, to clean up the column of objects
#       and prevent it from getting unmanageably large.  But the next version
#       of this script will implement a queue to blank out objects that have
#       scrolled far off the screen, so this requirement is pretty much obsolete.

from glife import *
import golly as g

import os
from os.path import join
from time import time

global display_step
global delay
global init_y
display_step=0
delay=.05

# remember initial hashing state so we can restore it if changed by a pattern file
inithash = g.getoption("hashing")

# ----------------------------------------------------------------

def ask (prompt):
   # prompt user and return entered string
   cursor1 = "_"
   cursor2 = ""

   g.show(prompt + " " + cursor1)
   inp = ""
   oldsecs = time()
   
   while True:
      newsecs = time()
      if newsecs - oldsecs >= 0.5:   # blink cursor each sec
         oldsecs = newsecs
         cursor1, cursor2 = cursor2, cursor1
         g.show(prompt + " " + inp + cursor1)

      ch = g.getkey()
      if len(ch) > 0:
         if ord(ch) == 13:    # return
         	g.show("")
         	return inp
         if ord(ch) == 8:     # backspace
            inp = inp[:-1]
            ch = ""
         inp += ch
         g.show(prompt + " " + inp + cursor1)

# ------------------------------------------------------------------------------

def handlekeys ():
  global display_step
  global delay
  ch = g.getkey()
  chret="<reset>"
  if len(ch) > 0:
    if ch==chr(28):
      x, y = g.getpos()
      g.setpos(str(int(x)-2**(4-g.getmag())),y)
    elif ch==chr(29):
      x, y = g.getpos()
      g.setpos(str(int(x)+2**(4-g.getmag())),y)
    elif ch==chr(30):
      x, y = g.getpos()
      g.setpos(x, str(int(y)-2**(4-g.getmag())))
    elif ch==chr(31):
      x, y = g.getpos()
      g.setpos(x, str(int(y)+2**(4-g.getmag())))
    elif ch=="f":
      g.fit()
    elif ch=="+":
      if delay==.025:
        display_step = display_step * 2 + 1 # this way it's always odd
      else:
        delay/=2
    elif ch=="-":
      if display_step != 0:
        display_step = (display_step - 1) / 2
      else:
        delay *= 2
    elif ch=="*":
      g.setmag( g.getmag() + 1 )
    elif ch=="/":
      g.setmag( g.getmag() - 1 )
    else:
    	chret=ch
    return chret

# ------------------------------------------------------------------------------

def slideshowmov ():
   basedir=ask("Enter SOF folder path or [Enter] for p2rotors: ")
   if basedir=="": basedir=g.appdir() + "Scripts" # assume p2rotors.sof stored with script
   for root, dirs, files in os.walk(basedir):
      for name in files:
         # ignore hidden files (like .DS_Store on Mac)
         if name.endswith(".sof") and not name.startswith("."):
            fullname = join(root, name)
            runSOF(fullname)      
      if "CVS" in dirs:
         dirs.remove("CVS")  # don't visit CVS directories

def runSOF (fname):
    g.show("space bar continues, arrow keys and / * - + f control pattern, q quits.")
    # need to open fname and call showSOF once on each line
    f=open(fname, 'r')
    index=0
    for line in f:
	index+=1
	# g.show(str(display_step))
	g.show(fname + ": #" + str(index))
	height = showSOF(line[line.rfind(",")+1:])
	# TODO: this section should be generalized for any speed, direction
	x, y = g.getpos()
	cur_y = int(y)
	max_y = cur_y + height + 4 # TODO: 3 should be the minimum safe distance for p2 objects (?)
	# g.show(str(max_y) + " :: " + str(cur_y))
	while cur_y<max_y:
		oldsecs = time()
		if display_step==0: g.run(1)
		else: g.run(display_step)
		# g.show(str(height) + "::" + str(cur_y) + "::" + str(max_y))
		if init_y<=cur_y: g.update()

		while display_step == 0:
			newsecs = time()
			if newsecs - oldsecs >= delay or init_y > cur_y: # other half of "horrible"
				break
		x, y = g.getpos()
		cur_y = int(y)
		if display_step==0: g.setpos(x,str(cur_y+1))
		else: g.setpos(x,str(cur_y+display_step))
		ch=handlekeys()
		if ch=="q": return

    f.close()
        
def showSOF (sofString):
    # process the SOF and produce an object below current view
    # TODO: pass in location to the function?

# temporary fix: draw SOF pattern sideways -- previous version below
#    x, y = g.getpos()
#    intx = int(x)
#    inty = int(y)
#    max_y+=inty
#    intx-=10 # TODO: keep track of number of lines and longest line
#    # maybe just return a pattern object and let the calling function handle placement
#    # (e.g., worrying about correcting for patterns that move...)
#    cur_x = intx
#    cellState=1
#    for ch in sofString:
#        if ch=='-':
#            cur_x=intx
#            inty+=1
#            cellState=1
#        elif ch=='.':
#            break
#        else:
#            runlength=ord(ch)-48
#            while runlength>0:
#                g.setcell(cur_x, inty, cellState)
#                cur_x+=1
#                runlength-=1
#            cellState=1-cellState

# temporary fix: draw SOF pattern sideways
    x, y = g.getpos()
    int_x = int(x) - 10
    # allow the Divine Hand of Golly to be seen at subpixel zooms
    int_y = int(y) + 256
    cur_y = int_y
    # TODO: keep track of number of lines -- 
    # maybe just return a pattern object and let the calling function handle placement
    # (e.g., worrying about correcting for patterns that move...)

    cellState=1
    max_y=int_y
    for ch in sofString:
        if ch=='-':
            if max_y<cur_y: max_y=cur_y
            cur_y=int_y
            int_x+=1
            cellState=1
        elif ch=='.':
            break
        else:
            runlength=ord(ch)-48
            while runlength>0:
                g.setcell(int_x, cur_y, cellState)
                cur_y+=1
                runlength-=1
            cellState=1-cellState
    return max_y-int_y
# ------------------------------------------------------------------------------

# show status bar so user sees messages
initstatus = g.setoption("showstatusbar", True)

# hide other stuff to maximize the viewport
# (don't switch to fullscreen because user won't see pattern name in title)
inittoolbar = g.setoption("showtoolbar", False)
initscripts = g.setoption("showscripts", False)
initpatterns = g.setoption("showpatterns", False)

g.new("HK Database Slide Show")
x, y = g.getpos()
init_y = int(y)+240 # horrible way to fudge this, but it works...
slideshowmov()
g.show("")

# restore original state
g.setoption("showstatusbar", initstatus)
g.setoption("showtoolbar", inittoolbar)
g.setoption("showscripts", initscripts)
g.setoption("showpatterns", initpatterns)


